@@ -43,7 +43,7 @@ Read the instructions on `source code repository
 """
 
 import logging
-from typing import List, Tuple, Optional, Union
+from typing import Callable, List, Tuple, Optional, Union
 
 import _pulsar
 
@@ -54,6 +54,7 @@ from _pulsar import Result, CompressionType, ConsumerType, InitialPosition, Part
 from pulsar.__about__ import __version__
 
 from pulsar.exceptions import *
+from pulsar.schema.schema import BytesSchema
 from pulsar.tableview import TableView
 
 from pulsar.functions.function import Function
@@ -131,12 +132,30 @@ class MessageId:
         return self._msg_id > other._msg_id
 
     @staticmethod
-    def deserialize(message_id_bytes):
+    def deserialize(message_id_bytes, topic: Optional[str] = None) -> _pulsar.MessageId:
         """
         Deserialize a message id object from a previously
         serialized bytes sequence.
+
+        Parameters
+        ----------
+        topic: str, optional
+            For multi-topics consumers, the topic name is required to deserialize the message id.
+
+            .. code-block:: python
+
+                msg = consumer.receive()
+                topic = msg.topic_name()
+                msg_id_bytes = msg.message_id().serialize()
+                # Store topic and msg_id_bytes somewhere
+                # Later, deserialize the message id
+                msg_id = MessageId.deserialize(msg_id_bytes, topic=topic)
+
         """
-        return _pulsar.MessageId.deserialize(message_id_bytes)
+        msg_id = _pulsar.MessageId.deserialize(message_id_bytes)
+        if topic is not None:
+            msg_id.topic_name(topic)
+        return msg_id
 
     @classmethod
     def wrap(cls, msg_id: _pulsar.MessageId):
@@ -228,6 +247,7 @@ class Message:
     @staticmethod
     def _wrap(_message):
         self = Message()
+        self._schema = BytesSchema()
         self._message = _message
         return self
 
@@ -678,6 +698,7 @@ class Client:
                         encryption_key=None,
                         crypto_key_reader: Union[None, CryptoKeyReader] = None,
                         access_mode: ProducerAccessMode = ProducerAccessMode.Shared,
+                        message_router: Callable[[Message, int], int]=None,
                         ):
         """
         Create a new producer on a given topic.
@@ -793,6 +814,10 @@ class Client:
             * WaitForExclusive: Producer creation is pending until it can acquire exclusive access.
             * ExclusiveWithFencing: Acquire exclusive access for the producer.
                                     Any existing producer will be removed and invalidated immediately.
+        message_router: optional
+            A custom message router function that takes a `Message` and the number of partitions
+            and returns the partition index to which the message should be routed. If not provided,
+            the default routing policy defined by `message_routing_mode` will be used.
         """
         _check_type(str, topic, 'topic')
         _check_type_or_none(str, producer_name, 'producer_name')
@@ -830,6 +855,10 @@ class Client:
         conf.chunking_enabled(chunking_enabled)
         conf.lazy_start_partitioned_producers(lazy_start_partitioned_producers)
         conf.access_mode(access_mode)
+        if message_router is not None:
+            underlying_router = lambda msg, num_partitions: int(message_router(Message._wrap(msg), num_partitions))
+            conf.message_router(underlying_router)
+
         if producer_name:
             conf.producer_name(producer_name)
         if initial_sequence_id:
@@ -1602,6 +1631,7 @@ class Consumer:
         for msg in msgs:
             m = Message()
             m._message = msg
+            m._schema = self._schema
             messages.append(m)
         return messages
 
